#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <richedit.h>
#include "AkelDLL.h"


//Functions prototypes
LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
HWND hMainWnd;
BOOL bOldWindows;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
WNDPROCDATA *NewMainProcData=NULL;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="LogDate";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Main(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);

  //Is plugin already loaded?
  if (bInitMain)
  {
    UninitMain();
  }
  else
  {
    InitMain();

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

LRESULT CALLBACK NewMainProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == AKDN_OPENDOCUMENT_FINISH)
  {
    EDITINFO ei;
    GETTEXTRANGE gtr;

    gtr.cpMin=0;
    gtr.cpMax=4;

    if (SendMessage(hMainWnd, AKD_GETTEXTRANGE, wParam, (LPARAM)&gtr))
    {
      if (bOldWindows)
      {
        char szFormat[MAX_PATH];
        char szDate[MAX_PATH];
        char szTime[MAX_PATH];

        if (!lstrcmpA((char *)gtr.pText, ".LOG"))
        {
          GetLocaleInfoA(LOCALE_USER_DEFAULT, LOCALE_SSHORTDATE, szFormat, MAX_PATH);
          GetDateFormatA(LOCALE_USER_DEFAULT, 0, NULL, szFormat, szDate, MAX_PATH);

          GetLocaleInfoA(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, szFormat, MAX_PATH);
          GetTimeFormatA(LOCALE_USER_DEFAULT, 0, NULL, szFormat, szTime, MAX_PATH);

          wsprintfA(szFormat, "\r%s %s\r", szTime, szDate);
          SendMessage((HWND)wParam, EM_SETSEL, 0x7FFFFFFF, 0x7FFFFFFF);

          if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
          {
            if (!ei.bReadOnly)
            {
              SendMessage(hMainWnd, AKD_REPLACESELA, wParam, (LPARAM)szFormat);
            }
            else MessageBeep(MB_OK);
          }
        }
      }
      else
      {
        wchar_t wszFormat[MAX_PATH];
        wchar_t wszDate[MAX_PATH];
        wchar_t wszTime[MAX_PATH];

        if (!lstrcmpW((wchar_t *)gtr.pText, L".LOG"))
        {
          GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_SSHORTDATE, wszFormat, MAX_PATH);
          GetDateFormatW(LOCALE_USER_DEFAULT, 0, NULL, wszFormat, wszDate, MAX_PATH);

          GetLocaleInfoW(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, wszFormat, MAX_PATH);
          GetTimeFormatW(LOCALE_USER_DEFAULT, 0, NULL, wszFormat, wszTime, MAX_PATH);

          wsprintfW(wszFormat, L"\r%s %s\r", wszTime, wszDate);
          SendMessage((HWND)wParam, EM_SETSEL, 0x7FFFFFFF, 0x7FFFFFFF);

          if (SendMessage(hMainWnd, AKD_GETEDITINFO, (WPARAM)NULL, (LPARAM)&ei))
          {
            if (!ei.bReadOnly)
            {
              SendMessage(hMainWnd, AKD_REPLACESELW, wParam, (LPARAM)wszFormat);
            }
            else MessageBeep(MB_OK);
          }
        }
      }
      SendMessage(hMainWnd, AKD_FREETEXT, 0, (LPARAM)gtr.pText);
    }
  }

  //Call next procedure
  return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hMainWnd=pd->hMainWnd;
  bOldWindows=pd->bOldWindows;
}

void InitMain()
{
  bInitMain=TRUE;

  //SubClass
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewMainProc, (LPARAM)&NewMainProcData);
}

void UninitMain()
{
  bInitMain=FALSE;

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
